import gurobi.*;
import java.util.stream.*;

public class Example {
    public static void main(String[] args) throws Exception {
        int[] values = new int[]{5, 2, 4, 2, 3, 6};
        int nBanknotes = values.length;

        // Create new environment.
        GRBEnv env = new GRBEnv();

        // Create empty optimization model.
        GRBModel model = new GRBModel(env);

        // Create variables x, y.
        // addVar(lowerBound, upperBound, objectiveCoeff, variableType, name)
        GRBVar[] x = new GRBVar[nBanknotes];
        for (int i = 0; i < nBanknotes; i++){
            x[i] =  model.addVar(0.0, 1.0, 0.0, GRB.INTEGER, "banknote" + i);
        }

        model.update();

        // Set objective: maximize Cmax
        GRBVar Cmax = model.addVar(0.0, GRB.INFINITY, 0, GRB.INTEGER, "Cmax");

        // Integrate new variables into model.
        model.update();

        GRBLinExpr obj = new GRBLinExpr();
        obj.addTerm(1.0, Cmax);
        model.setObjective(obj, GRB.MINIMIZE);

        // Add constraints:
        GRBLinExpr cons1 = new GRBLinExpr();
        for (int i = 0; i < nBanknotes; i++) {
            cons1.addTerm(values[i], x[i]);
        }
        cons1.addTerm(-1.0, Cmax);
        // addConstr(leftHandSide, inequalityType, rightHandSide, name)
        model.addConstr(cons1, GRB.LESS_EQUAL, 0, "cons1");

        GRBLinExpr cons2 = new GRBLinExpr();
        for (int i = 0; i < nBanknotes; i++) {
            cons2.addTerm(-values[i], x[i]);
        }
        cons2.addTerm(-1.0, Cmax);
        // addConstr(leftHandSide, inequalityType, rightHandSide, name)
        model.addConstr(cons2, GRB.LESS_EQUAL, -IntStream.of(values).sum(), "cons2");

        // Solve the model.
        model.optimize();

        // Print the objective
        // and the values of the decision variables in the solution.
        for (int i = 0; i < nBanknotes; i++){
            System.out.println(x[i].get(GRB.StringAttr.VarName)+ " " + x[i].get(GRB.DoubleAttr.X));
        }
        System.out.println("Obj: " + model.get(GRB.DoubleAttr.ObjVal));
    }
}
