#include <gurobi_c++.h>
#include <vector>
using namespace std;

// Maximum independent set.

int main(int argc, char *argv[]) {
  // Parameters.
  // Graph taken from https://www.tutorialspoint.com/graph_theory/graph_theory_independent_sets.htm
  // (see Maximum Independent Vertex Set section)
  int numVertices = 8;
  vector<pair<int, int>> edges = {
    {0, 1}, {0, 4}, {1, 4}, {1, 2}, {2, 5}, {2, 3}, {3, 5}, {4, 5}, {4, 6}, {5, 7}, {6, 7}
  };

  GRBEnv env;
  GRBModel model(env);

  // For each vertex, create a binary variable denoting whether it is in the maximum independent set.
  // We will index the vertices from 0.
  GRBVar x[numVertices];
  for (int i = 0; i < numVertices; i++) {
    x[i] = model.addVar(0, 1, 0.0, GRB_INTEGER);
  }

  model.update();

  // Constraint: For each edge, make sure that at most one vertex is in the maximum independent set.
  {
    for (auto &edge: edges) {
      model.addConstr(x[edge.first] + x[edge.second] <= 1);
    }
  }

  // Objective: maximize the number of the vertices in the independent set.
  {
    GRBLinExpr objfn = 0;
    for (int i = 0; i < numVertices; i++) {
      objfn += x[i];
    }
    model.setObjective(objfn, GRB_MAXIMIZE);
  }

  model.optimize();

  if (model.get(GRB_IntAttr_Status) == GRB_OPTIMAL) {
    cout << "Optimal objective value: " << model.get(GRB_DoubleAttr_ObjVal) << endl;
    for (int i = 0; i < numVertices; i++) {
      cout << "Vertex " << i << " in the maximum independent set? " << x[i].get(GRB_DoubleAttr_X) << endl;
    }
  }

  return 0;
}
